@@ -64,17 +64,12 @@ bool CodeGenerator::GenerateAll(const std::vector<const FileDescriptor*>& files,
 
 absl::StatusOr<FeatureSetDefaults> CodeGenerator::BuildFeatureSetDefaults()
     const {
-  if ((GetSupportedFeatures() & FEATURE_SUPPORTS_EDITIONS) == 0) {
-    // For generators that don't fully support editions yet, provide an
-    // optimistic set of defaults.  Protoc will check this condition later
-    // anyway.
-    return FeatureResolver::CompileDefaults(
-        FeatureSet::descriptor(), GetFeatureExtensions(),
-        PROTOBUF_MINIMUM_EDITION, PROTOBUF_MAXIMUM_EDITION);
-  }
+  // For generators that don't fully support editions yet, provide an
+  // optimistic set of defaults.  Protoc will check this condition later
+  // anyway.
   return FeatureResolver::CompileDefaults(
-      FeatureSet::descriptor(), GetFeatureExtensions(), GetMinimumEdition(),
-      GetMaximumEdition());
+      FeatureSet::descriptor(), GetFeatureExtensions(), ProtocMinimumEdition(),
+      MaximumKnownEdition());
 }
 
 GeneratorContext::~GeneratorContext() = default;
@@ -108,44 +103,10 @@ void GeneratorContext::GetCompilerVersion(Version* version) const {
   version->set_suffix(GOOGLE_PROTOBUF_VERSION_SUFFIX);
 }
 
-// Parses a set of comma-delimited name/value pairs.
-void ParseGeneratorParameter(
-    absl::string_view text,
-    std::vector<std::pair<std::string, std::string> >* output) {
-  std::vector<absl::string_view> parts =
-      absl::StrSplit(text, ',', absl::SkipEmpty());
-
-  for (absl::string_view part : parts) {
-    auto equals_pos = part.find_first_of('=');
-    if (equals_pos == absl::string_view::npos) {
-      output->emplace_back(part, "");
-    } else {
-      output->emplace_back(part.substr(0, equals_pos),
-                           part.substr(equals_pos + 1));
-    }
-  }
-}
-
-// Strips ".proto" or ".protodevel" from the end of a filename.
-std::string StripProto(absl::string_view filename) {
-  if (absl::EndsWith(filename, ".protodevel")) {
-    return std::string(absl::StripSuffix(filename, ".protodevel"));
-  } else {
-    return std::string(absl::StripSuffix(filename, ".proto"));
-  }
-}
-
-bool IsKnownFeatureProto(absl::string_view filename) {
-  if (filename == "google/protobuf/cpp_features.proto" ||
-      filename == "google/protobuf/java_features.proto") {
-    return true;
-  }
-  return false;
-}
-
 bool CanSkipEditionCheck(absl::string_view filename) {
   return absl::StartsWith(filename, "google/protobuf/") ||
-         absl::StartsWith(filename, "upb/");
+         absl::StartsWith(filename, "upb/") ||
+         absl::StartsWith(filename, "com/google/protobuf/");
 }
 
 }  // namespace compiler
