@@ -8,6 +8,7 @@
 #include "google/protobuf/compiler/rust/naming.h"
 
 #include <algorithm>
+#include <cstddef>
 #include <string>
 #include <vector>
 
@@ -16,17 +17,18 @@
 #include "absl/strings/ascii.h"
 #include "absl/strings/match.h"
 #include "absl/strings/str_cat.h"
+#include "absl/strings/str_format.h"
 #include "absl/strings/str_join.h"
 #include "absl/strings/str_replace.h"
 #include "absl/strings/string_view.h"
 #include "absl/strings/strip.h"
-#include "absl/strings/substitute.h"
 #include "google/protobuf/compiler/code_generator.h"
 #include "google/protobuf/compiler/cpp/helpers.h"
 #include "google/protobuf/compiler/rust/context.h"
 #include "google/protobuf/compiler/rust/rust_field_type.h"
 #include "google/protobuf/compiler/rust/rust_keywords.h"
 #include "google/protobuf/descriptor.h"
+#include "google/protobuf/port.h"
 
 // Must be included last.
 #include "google/protobuf/port_def.inc"
@@ -37,7 +39,19 @@ namespace compiler {
 namespace rust {
 
 std::string GetCrateName(Context& ctx, const FileDescriptor& dep) {
-  return RsSafeName(ctx.generator_context().ImportPathToCrateName(dep.name()));
+  std::string crate_name = RsSafeName(ctx.ImportPathToCrateName(dep.name()));
+  if (absl::StartsWith(crate_name, "crate::")) {
+    return crate_name;
+  }
+  return absl::StrCat("::", crate_name);
+}
+
+std::string GetEntryPointRsFilePath(Context& ctx, const FileDescriptor& file) {
+  size_t last_slash = file.name().find_last_of('/');
+  return absl::StrCat(last_slash == std::string::npos
+                          ? ""
+                          : file.name().substr(0, last_slash + 1),
+                      ctx.opts().generated_entry_point_rs_file_name);
 }
 
 std::string GetRsFile(Context& ctx, const FileDescriptor& file) {
@@ -60,112 +74,46 @@ std::string GetThunkCcFile(Context& ctx, const FileDescriptor& file) {
 
 std::string GetHeaderFile(Context& ctx, const FileDescriptor& file) {
   auto basename = StripProto(file.name());
-  return absl::StrCat(basename, ".proto.h");
+  constexpr absl::string_view kCcGencodeExt = ".pb.h";
+
+  return absl::StrCat(basename, kCcGencodeExt);
 }
 
 std::string RawMapThunk(Context& ctx, const Descriptor& msg,
                         absl::string_view key_t, absl::string_view op) {
-  return absl::StrCat("__rust_proto_thunk__Map_", key_t, "_",
+  return absl::StrCat("proto2_rust_thunk_Map_", key_t, "_",
                       GetUnderscoreDelimitedFullName(ctx, *&msg), "_", op);
 }
 
 std::string RawMapThunk(Context& ctx, const EnumDescriptor& desc,
                         absl::string_view key_t, absl::string_view op) {
-  return absl::StrCat("__rust_proto_thunk__Map_", key_t, "_",
-                      GetUnderscoreDelimitedFullName(ctx, *&desc), "_", op);
+  // Enums are always 32 bits.
+  return absl::StrCat("proto2_rust_thunk_Map_", key_t, "_i32_", op);
 }
 
-namespace {
-
-template <typename T>
-std::string FieldPrefix(Context& ctx, const T& field) {
-  // NOTE: When ctx.is_upb(), this functions outputs must match the symbols
-  // that the upbc plugin generates exactly. Failure to do so correctly results
-  // in a link-time failure.
-  absl::string_view prefix = ctx.is_cpp() ? "__rust_proto_thunk__" : "";
-  std::string thunk_prefix = absl::StrCat(
-      prefix, GetUnderscoreDelimitedFullName(ctx, *field.containing_type()));
-  return thunk_prefix;
-}
-
-template <typename T>
-std::string ThunkName(Context& ctx, const T& field, absl::string_view op) {
-  std::string thunkName = FieldPrefix(ctx, field);
-
-  absl::string_view format;
-  if (ctx.is_upb() && op == "get") {
-    // upb getter is simply the field name (no "get" in the name).
-    format = "_$1";
-  } else if (ctx.is_upb() && op == "get_mut") {
-    // same as above, with with `mutable` prefix
-    format = "_mutable_$1";
-  } else if (ctx.is_upb() && op == "case") {
-    // some upb functions are in the order x_op compared to has/set/clear which
-    // are in the other order e.g. op_x.
-    format = "_$1_$0";
-  } else {
-    format = "_$0_$1";
-  }
-
-  absl::SubstituteAndAppend(&thunkName, format, op, field.name());
-  return thunkName;
-}
-
-std::string ThunkMapOrRepeated(Context& ctx, const FieldDescriptor& field,
-                               absl::string_view op) {
-  if (!ctx.is_upb()) {
-    return ThunkName<FieldDescriptor>(ctx, field, op);
-  }
-
-  std::string thunkName = absl::StrCat("_", FieldPrefix(ctx, field));
-  absl::string_view format;
-  if (op == "get") {
-    format = field.is_map() ? "_$1_upb_map" : "_$1_upb_array";
-  } else if (op == "get_mut") {
-    format = field.is_map() ? "_$1_mutable_upb_map" : "_$1_mutable_upb_array";
-  } else {
-    return ThunkName<FieldDescriptor>(ctx, field, op);
-  }
-
-  absl::SubstituteAndAppend(&thunkName, format, op, field.name());
-  return thunkName;
-}
-
-}  // namespace
-
 std::string ThunkName(Context& ctx, const FieldDescriptor& field,
                       absl::string_view op) {
-  if (field.is_map() || field.is_repeated()) {
-    return ThunkMapOrRepeated(ctx, field, op);
-  }
-  return ThunkName<FieldDescriptor>(ctx, field, op);
+  ABSL_CHECK(ctx.is_cpp());
+  return absl::StrCat("proto2_rust_thunk_",
+                      UnderscoreDelimitFullName(ctx, field.full_name()), "_",
+                      op);
 }
 
 std::string ThunkName(Context& ctx, const OneofDescriptor& field,
                       absl::string_view op) {
-  return ThunkName<OneofDescriptor>(ctx, field, op);
+  ABSL_CHECK(ctx.is_cpp());
+  return absl::StrCat("proto2_rust_thunk_",
+                      UnderscoreDelimitFullName(ctx, field.full_name()), "_",
+                      op);
 }
 
 std::string ThunkName(Context& ctx, const Descriptor& msg,
                       absl::string_view op) {
-  absl::string_view prefix = ctx.is_cpp() ? "__rust_proto_thunk__" : "";
+  absl::string_view prefix = ctx.is_cpp() ? "proto2_rust_thunk_Message_" : "";
   return absl::StrCat(prefix, GetUnderscoreDelimitedFullName(ctx, msg), "_",
                       op);
 }
 
-std::string VTableName(const FieldDescriptor& field) {
-  return absl::StrCat("__", absl::AsciiStrToUpper(field.name()), "_VTABLE");
-}
-
-template <typename Desc>
-std::string GetFullyQualifiedPath(Context& ctx, const Desc& desc) {
-  auto rel_path = GetCrateRelativeQualifiedPath(ctx, desc);
-  if (IsInCurrentlyGeneratingCrate(ctx, desc)) {
-    return absl::StrCat("crate::", rel_path);
-  }
-  return absl::StrCat(GetCrateName(ctx, *desc.file()), "::", rel_path);
-}
-
 template <typename Desc>
 std::string GetUnderscoreDelimitedFullName(Context& ctx, const Desc& desc) {
   return UnderscoreDelimitFullName(ctx, desc.full_name());
@@ -195,19 +143,60 @@ std::string RsTypePath(Context& ctx, const FieldDescriptor& field) {
     case RustFieldType::DOUBLE:
       return "f64";
     case RustFieldType::BYTES:
-      return "[u8]";
+      return "::protobuf::ProtoBytes";
     case RustFieldType::STRING:
-      return "::__pb::ProtoStr";
+      return "::protobuf::ProtoString";
     case RustFieldType::MESSAGE:
-      return GetFullyQualifiedPath(ctx, *field.message_type());
+      return RsTypePath(ctx, *field.message_type());
     case RustFieldType::ENUM:
-      return GetFullyQualifiedPath(ctx, *field.enum_type());
+      return RsTypePath(ctx, *field.enum_type());
   }
-  ABSL_LOG(FATAL) << "Unsupported field type: " << field.type_name();
+  ABSL_LOG(ERROR) << "Unknown field type: " << field.type_name();
+  internal::Unreachable();
 }
 
-std::string RustModuleForContainingType(Context& ctx,
-                                        const Descriptor* containing_type) {
+std::string RsTypePath(Context& ctx, const Descriptor& message) {
+  return absl::StrCat(RustModule(ctx, message), RsSafeName(message.name()));
+}
+
+std::string RsTypePath(Context& ctx, const EnumDescriptor& descriptor) {
+  return absl::StrCat(RustModule(ctx, descriptor), EnumRsName(descriptor));
+}
+
+std::string RsViewType(Context& ctx, const FieldDescriptor& field,
+                       absl::string_view lifetime) {
+  switch (GetRustFieldType(field)) {
+    case RustFieldType::BOOL:
+    case RustFieldType::INT32:
+    case RustFieldType::INT64:
+    case RustFieldType::UINT32:
+    case RustFieldType::UINT64:
+    case RustFieldType::FLOAT:
+    case RustFieldType::DOUBLE:
+    case RustFieldType::ENUM:
+      // The View type of all scalars and enums can be spelled as the type
+      // itself.
+      return RsTypePath(ctx, field);
+    case RustFieldType::BYTES:
+      return absl::StrFormat("&%s [u8]", lifetime);
+    case RustFieldType::STRING:
+      return absl::StrFormat("&%s ::protobuf::ProtoStr", lifetime);
+    case RustFieldType::MESSAGE:
+      if (lifetime.empty()) {
+        return absl::StrFormat("%sView",
+                               RsTypePath(ctx, *field.message_type()));
+      } else {
+        return absl::StrFormat(
+            "%sView<%s>", RsTypePath(ctx, *field.message_type()), lifetime);
+      }
+  }
+  ABSL_LOG(FATAL) << "Unsupported field type: " << field.type_name();
+  internal::Unreachable();
+}
+
+static std::string RustModuleForContainingType(
+    Context& ctx, const Descriptor* containing_type,
+    const FileDescriptor& file) {
   std::vector<std::string> modules;
 
   // Innermost to outermost order.
@@ -226,33 +215,39 @@ std::string RustModuleForContainingType(Context& ctx,
     modules.push_back("");
   }
 
-  return absl::StrJoin(modules, "::");
+  std::string crate_relative = absl::StrJoin(modules, "::");
+
+  if (IsInCurrentlyGeneratingCrate(ctx, file)) {
+    std::string prefix;
+    for (size_t i = 0; i < ctx.GetModuleDepth(); ++i) {
+      prefix += "super::";
+    }
+    return absl::StrCat(prefix, crate_relative);
+  }
+  return absl::StrCat(GetCrateName(ctx, file), "::", crate_relative);
 }
 
 std::string RustModule(Context& ctx, const Descriptor& msg) {
-  return RustModuleForContainingType(ctx, msg.containing_type());
+  return RustModuleForContainingType(ctx, msg.containing_type(), *msg.file());
 }
 
 std::string RustModule(Context& ctx, const EnumDescriptor& enum_) {
-  return RustModuleForContainingType(ctx, enum_.containing_type());
+  return RustModuleForContainingType(ctx, enum_.containing_type(),
+                                     *enum_.file());
 }
 
 std::string RustModule(Context& ctx, const OneofDescriptor& oneof) {
-  return RustModuleForContainingType(ctx, oneof.containing_type());
+  return RustModuleForContainingType(ctx, oneof.containing_type(),
+                                     *oneof.file());
 }
 
-std::string RustInternalModuleName(Context& ctx, const FileDescriptor& file) {
+std::string RustInternalModuleName(const FileDescriptor& file) {
   return RsSafeName(
-      absl::StrReplaceAll(StripProto(file.name()), {{"_", "__"}, {"/", "_s"}}));
-}
-
-std::string GetCrateRelativeQualifiedPath(Context& ctx, const Descriptor& msg) {
-  return absl::StrCat(RustModule(ctx, msg), RsSafeName(msg.name()));
-}
-
-std::string GetCrateRelativeQualifiedPath(Context& ctx,
-                                          const EnumDescriptor& enum_) {
-  return absl::StrCat(RustModule(ctx, enum_), EnumRsName(enum_));
+      absl::StrReplaceAll(StripProto(file.name()), {
+                                                       {"_", "__"},
+                                                       {"/", "_s"},
+                                                       {"-", "__"},
+                                                   }));
 }
 
 std::string FieldInfoComment(Context& ctx, const FieldDescriptor& field) {
@@ -270,6 +265,40 @@ std::string FieldInfoComment(Context& ctx, const FieldDescriptor& field) {
   return comment;
 }
 
+static constexpr absl::string_view kAccessorPrefixes[] = {"clear_", "has_",
+                                                          "set_"};
+
+static constexpr absl::string_view kAccessorSuffixes[] = {"_mut", "_opt"};
+
+std::string FieldNameWithCollisionAvoidance(const FieldDescriptor& field) {
+  absl::string_view name = field.name();
+  const Descriptor& msg = *field.containing_type();
+
+  for (absl::string_view prefix : kAccessorPrefixes) {
+    if (absl::StartsWith(name, prefix)) {
+      absl::string_view without_prefix = name;
+      without_prefix.remove_prefix(prefix.size());
+
+      if (msg.FindFieldByName(without_prefix) != nullptr) {
+        return absl::StrCat(name, "_", field.number());
+      }
+    }
+  }
+
+  for (absl::string_view suffix : kAccessorSuffixes) {
+    if (absl::EndsWith(name, suffix)) {
+      absl::string_view without_suffix = name;
+      without_suffix.remove_suffix(suffix.size());
+
+      if (msg.FindFieldByName(without_suffix) != nullptr) {
+        return absl::StrCat(name, "_", field.number());
+      }
+    }
+  }
+
+  return std::string(name);
+}
+
 std::string RsSafeName(absl::string_view name) {
   if (!IsLegalRawIdentifierName(name)) {
     return absl::StrCat(name,
@@ -309,12 +338,14 @@ std::string EnumValueRsName(const MultiCasePrefixStripper& stripper,
 }
 
 std::string OneofViewEnumRsName(const OneofDescriptor& oneof) {
-  return RsSafeName(SnakeToUpperCamelCase(oneof.name()));
+  return SnakeToUpperCamelCase(oneof.name()) + "Oneof";
 }
 
 std::string OneofCaseEnumRsName(const OneofDescriptor& oneof) {
-  // Note: This is the name used for the cpp Case enum, we use it for both
-  // the Rust Case enum as well as for the cpp case enum in the cpp thunk.
+  return SnakeToUpperCamelCase(oneof.name()) + "Case";
+}
+
+std::string OneofCaseEnumCppName(const OneofDescriptor& oneof) {
   return SnakeToUpperCamelCase(oneof.name()) + "Case";
 }
 
@@ -365,6 +396,21 @@ std::string ScreamingSnakeToUpperCamelCase(absl::string_view input) {
   return result;
 }
 
+std::string CrubitCcSymbolName(const Descriptor& msg) {
+  // To support forward declares of C++ types, Crubit requires that the symbol
+  // literal is spelled identical to the one used in the generated bindings.
+  // This requires some string mangling here to make them match.
+  std::string cpp_name = cpp::QualifiedClassName(&msg);
+  if (absl::StartsWith(cpp_name, "::")) {
+    cpp_name = cpp_name.substr(2);
+  }
+  cpp_name = absl::StrReplaceAll(cpp_name,
+                                 {{"::", " :: "}, {"<", " < "}, {">", " > "}});
+  absl::StripTrailingAsciiWhitespace(&cpp_name);
+
+  return cpp_name;
+}
+
 MultiCasePrefixStripper::MultiCasePrefixStripper(absl::string_view prefix)
     : prefixes_{
           std::string(prefix),
@@ -393,45 +439,9 @@ absl::string_view MultiCasePrefixStripper::StripPrefix(
   return name;
 }
 
-PROTOBUF_CONSTINIT const MapKeyType kMapKeyTypes[] = {
-    {/*thunk_ident=*/"i32", /*rs_key_t=*/"i32", /*rs_ffi_key_t=*/"i32",
-     /*rs_to_ffi_key_expr=*/"key", /*rs_from_ffi_key_expr=*/"ffi_key",
-     /*cc_key_t=*/"int32_t", /*cc_ffi_key_t=*/"int32_t",
-     /*cc_from_ffi_key_expr=*/"key",
-     /*cc_to_ffi_key_expr=*/"cpp_key"},
-    {/*thunk_ident=*/"u32", /*rs_key_t=*/"u32", /*rs_ffi_key_t=*/"u32",
-     /*rs_to_ffi_key_expr=*/"key", /*rs_from_ffi_key_expr=*/"ffi_key",
-     /*cc_key_t=*/"uint32_t", /*cc_ffi_key_t=*/"uint32_t",
-     /*cc_from_ffi_key_expr=*/"key",
-     /*cc_to_ffi_key_expr=*/"cpp_key"},
-    {/*thunk_ident=*/"i64", /*rs_key_t=*/"i64", /*rs_ffi_key_t=*/"i64",
-     /*rs_to_ffi_key_expr=*/"key", /*rs_from_ffi_key_expr=*/"ffi_key",
-     /*cc_key_t=*/"int64_t", /*cc_ffi_key_t=*/"int64_t",
-     /*cc_from_ffi_key_expr=*/"key",
-     /*cc_to_ffi_key_expr=*/"cpp_key"},
-    {/*thunk_ident=*/"u64", /*rs_key_t=*/"u64", /*rs_ffi_key_t=*/"u64",
-     /*rs_to_ffi_key_expr=*/"key", /*rs_from_ffi_key_expr=*/"ffi_key",
-     /*cc_key_t=*/"uint64_t", /*cc_ffi_key_t=*/"uint64_t",
-     /*cc_from_ffi_key_expr=*/"key",
-     /*cc_to_ffi_key_expr=*/"cpp_key"},
-    {/*thunk_ident=*/"bool", /*rs_key_t=*/"bool", /*rs_ffi_key_t=*/"bool",
-     /*rs_to_ffi_key_expr=*/"key", /*rs_from_ffi_key_expr=*/"ffi_key",
-     /*cc_key_t=*/"bool", /*cc_ffi_key_t=*/"bool",
-     /*cc_from_ffi_key_expr=*/"key",
-     /*cc_to_ffi_key_expr=*/"cpp_key"},
-    {/*thunk_ident=*/"ProtoStr",
-     /*rs_key_t=*/"$pb$::ProtoStr",
-     /*rs_ffi_key_t=*/"$pbr$::PtrAndLen",
-     /*rs_to_ffi_key_expr=*/"key.as_bytes().into()",
-     /*rs_from_ffi_key_expr=*/
-     "$pb$::ProtoStr::from_utf8_unchecked(ffi_key.as_ref())",
-     /*cc_key_t=*/"std::string",
-     /*cc_ffi_key_t=*/"google::protobuf::rust_internal::PtrAndLen",
-     /*cc_from_ffi_key_expr=*/
-     "std::string(key.ptr, key.len)", /*cc_to_ffi_key_expr=*/
-     "google::protobuf::rust_internal::PtrAndLen(cpp_key.data(), cpp_key.size())"}};
-
 }  // namespace rust
 }  // namespace compiler
 }  // namespace protobuf
 }  // namespace google
+
+#include "google/protobuf/port_undef.inc"
